from public.dde_desktop_public_widget.dde_desktop_public_widget import DdeDesktopPublicWidget
from public.dde_dock_public_widget.dde_dock_public_widget import DdeDockPublicWidget
from public.dde_launcher_public_widget.dde_launcher_public_widget import DdeLauncherPublicWidget
from public.open_mode_public_widget.open_mode_public_widget import OpenModePublicWidget
from public.other_public_widget.deepin_compress_public_widget import DeepinCompressPublicWidget
from public.other_public_widget.deepin_manual_public_widget import DeepinManualPublicWidget
from public.right_menu_public_widget.right_menu_public_widget import RightMenuPublicWidget
